@@ -67,8 +67,10 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
         z <- as.matrix(z)
     if (nrow(x) != N || nrow(z) != N)
         stop("both 'x' and 'z' must have the same number of rows as 'y'")
-    x <- `colnames<-`(model.matrix(~0 + x), colnames(x))
-    z <- `colnames<-`(model.matrix(~0 + z), colnames(z))
+    p <- ncol(x)
+    q <- ncol(z)
+    x <- `colnames<-`(model.matrix(~ 0 + x), colnames(x) %||% paste("x", 1:p, sep = ""))
+    z <- `colnames<-`(model.matrix(~ 0 + z), colnames(z) %||% paste("z", 1:q, sep = ""))
 
     # check beta_set and sign_set
     stopifnot(beta_set %in% 1:J, is.logical(sign_set))
@@ -78,31 +80,28 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
         K = 21, C = 5)  # control parameters
     con[names(control)] <- control
 
-    # dimensions, response categories, etc.
-    p <- ncol(x)
-    q <- ncol(z)
-    y_names <- names(y)
-    x_names <- colnames(x)
-    z_names <- colnames(z)
+    # set environments for utility functions
     environment(loglik_ltm) <- environment(theta_post_ltm) <- environment(dummy_fun_ltm) <- environment(tab2df_ltm) <- environment()
 
-    # GH points
+    # GL points
     K <- con[["K"]]
     theta_ls <- con[["C"]] * GLpoints[[K]][["x"]]
     qw_ls <- con[["C"]] * GLpoints[[K]][["w"]]
 
     # initialization
+    lm_opr <- tcrossprod(solve(crossprod(x)), x)
     theta_eap <- {
         tmp <- rowMeans(y, na.rm = TRUE)
         (tmp - mean(tmp, na.rm = TRUE))/sd(tmp, na.rm = TRUE)
     }
     theta_eap[is.na(theta_eap)] <- 0
-
     alpha <- rep(0, J)
     beta <- vapply(y, function(y) cov(y, theta_eap, use = "complete.obs")/var(theta_eap),
         numeric(1L))
-    gamma <- solve(t(x) %*% x) %*% t(x) %*% theta_eap
+    gamma <- lm_opr %*% theta_eap
     lambda <- rep(0, q)
+    fitted_mean <- as.vector(x %*% gamma)
+    fitted_var <- rep(1, N)
 
     # EM algorithm
     for (iter in seq(1, con[["max_iter"]])) {
@@ -131,25 +130,28 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
         theta_vap <- t(theta_ls^2 %*% w) - theta_eap^2
 
         # variance regression
-        gamma <- solve(t(x) %*% x) %*% t(x) %*% theta_eap
+        gamma <- lm_opr %*% theta_eap
         r2 <- (theta_eap - x %*% gamma)^2 + theta_vap
-        s2 <- glm.fit(x = z, y = r2, intercept = FALSE, family = Gamma(link = "log"))[["fitted.values"]]
-        loglik <- -0.5 * (log(s2) + r2/s2)
-        LL0 <- sum(loglik)
-        dLL <- 1
-        for (m in seq(1, con[["max_iter2"]])) {
-            gamma <- lm(theta_eap ~ 0 + x, weights = 1/s2)[["coefficients"]]
-            r2 <- (theta_eap - x %*% gamma)^2 + theta_vap
-            var_reg <- glm.fit(x = z, y = r2, intercept = FALSE, family = Gamma(link = "log"))
-            s2 <- var_reg[["fitted.values"]]
-            loglik <- -0.5 * (log(s2) + r2/s2)
-            LL_temp <- sum(loglik)
-            dLL <- LL_temp - LL0
-            if (dLL < con[["eps2"]])
-                break
-            LL0 <- LL_temp
+
+        if (ncol(z)==1) lambda <- log(mean(r2)) else{
+          s2 <- glm.fit(x = z, y = r2, intercept = FALSE, family = Gamma(link = "log"))[["fitted.values"]]
+          loglik <- -0.5 * (log(s2) + r2/s2)
+          LL0 <- sum(loglik)
+          dLL <- 1
+          for (m in seq(1, con[["max_iter2"]])) {
+              gamma <- lm.wfit(x, theta_eap, w = 1/s2)[["coefficients"]]
+              r2 <- (theta_eap - x %*% gamma)^2 + theta_vap
+              var_reg <- glm.fit(x = z, y = r2, intercept = FALSE, family = Gamma(link = "log"))
+              s2 <- var_reg[["fitted.values"]]
+              loglik <- -0.5 * (log(s2) + r2/s2)
+              LL_temp <- sum(loglik)
+              dLL <- LL_temp - LL0
+              if (dLL < con[["eps2"]])
+                  break
+              LL0 <- LL_temp
+          }
+          lambda <- var_reg[["coefficients"]]
         }
-        lambda <- var_reg[["coefficients"]]
 
         # location constraint
         tmp <- mean(x %*% gamma)
@@ -167,6 +169,8 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
             gamma <- -gamma
             beta <- -beta
         }
+        fitted_mean <- as.vector(x %*% gamma)
+        fitted_var <- exp(as.vector(z %*% lambda))
         cat(".")
 
         # check convergence
@@ -182,12 +186,11 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
     }
 
     # inference
-    pik <- matrix(unlist(Map(pryr::partial(dnorm, x = theta_ls), mean = as.vector(x %*%
-        gamma), sd = as.vector(exp(z %*% lambda)))), N, K, byrow = TRUE) *
-        matrix(qw_ls, N, K, byrow = TRUE)
+    pik <- matrix(unlist(Map(partial(dnorm, x = theta_ls), mean = fitted_mean,
+                             sd = sqrt(fitted_var))), N, K, byrow = TRUE) * matrix(qw_ls, N, K, byrow = TRUE)
     Lijk <- lapply(theta_ls, function(theta_k) exp(loglik_ltm(alpha = alpha,
         beta = beta, rep(theta_k, N))))  # K-list
-    Lik <- vapply(Lijk, pryr::compose(exp, pryr::partial(rowSums, na.rm = TRUE),
+    Lik <- vapply(Lijk, compose(exp, partial(rowSums, na.rm = TRUE),
         log), numeric(N))
     Li <- rowSums(Lik * pik)
 
@@ -206,7 +209,7 @@ hltm <- function(y, x = matrix(1, nrow(y), 1), z = matrix(1, nrow(y), 1),
         s_lambda <- vapply(1:N, si_lambda, numeric(q - 1))
     s_all <- rbind(t(s_ab), s_gamma, s_lambda)
     s_all[is.na(s_all)] <- 0
-    covmat <- solve(s_all %*% t(s_all))
+    covmat <- solve(tcrossprod(s_all))
     se_all <- sqrt(diag(covmat))
 
     # reorganize se_all
